@@ -22,10 +22,9 @@ from marshmallow.fields import (  # noqa
     Integer,
     Decimal,
     Boolean,
-    FormattedString,
     Float,
     DateTime,
-    LocalDateTime,
+    # LocalDateTime,
     Time,
     Date,
     TimeDelta,
@@ -57,7 +56,7 @@ class Nested(FieldNested):
 
         it is the only way to propagate the context at each call
         """
-        self.__schema = None
+        self._schema = None
         return super(Nested, self).schema
 
     def _deserialize(self, value, attr, data, **kwargs):
@@ -219,10 +218,9 @@ class Country(String):
 
     def _deserialize(self, value, attr, data, **kwargs):
         if value is not None:
-            try:
-                import pycountry
-                return pycountry.countries.get(alpha_3=value)
-            except Exception:
+            import pycountry
+            value = pycountry.countries.get(alpha_3=value)
+            if value is None:
                 raise ValidationError('Not a valid country.')
 
         return value
@@ -307,7 +305,7 @@ class Color(String):
 
         return value
 
-    def _deserialize(self, value, attr, data):
+    def _deserialize(self, value, attr, data, **kwargs):
         if value is not None:
             try:
                 from colour import Color
