@@ -6,47 +6,15 @@
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file,You can
 # obtain one at http://mozilla.org/MPL/2.0/.
-from anyblok.tests.testcase import DBTestCase
+import pytest
+from .conftest import init_registry
 from anyblok_marshmallow import SchemaWrapper
 from anyblok import Declarations
 from anyblok.column import Integer
 from anyblok.relationship import One2One, Many2One, Many2Many
 
 
-class TestRelationShip(DBTestCase):
-
-    def add_field_one2one(self):
-
-        @Declarations.register(Declarations.Model)
-        class Exemple:
-            id = Integer(primary_key=True)
-
-        @Declarations.register(Declarations.Model)
-        class Exemple2:
-            id = Integer(primary_key=True)
-            exemple = One2One(model='Model.Exemple', backref='exemple2')
-
-    def add_field_one2many(self):
-
-        @Declarations.register(Declarations.Model)
-        class Exemple:
-            id = Integer(primary_key=True)
-
-        @Declarations.register(Declarations.Model)
-        class Exemple2:
-            id = Integer(primary_key=True)
-            exemple = Many2One(model='Model.Exemple', one2many='exemple2')
-
-    def add_field_many2many(self):
-
-        @Declarations.register(Declarations.Model)
-        class Exemple:
-            id = Integer(primary_key=True)
-
-        @Declarations.register(Declarations.Model)
-        class Exemple2:
-            id = Integer(primary_key=True)
-            exemple = Many2Many(model='Model.Exemple', many2many='exemple2')
+class SchemaBehaviour:
 
     def getExempleSchema(self):
 
@@ -62,14 +30,41 @@ class TestRelationShip(DBTestCase):
 
         return Exemple2Schema
 
-    def test_dump_one2one_1(self):
-        registry = self.init_registry(self.add_field_one2one)
+
+def add_field_one2one():
+
+    @Declarations.register(Declarations.Model)
+    class Exemple:
+        id = Integer(primary_key=True)
+
+    @Declarations.register(Declarations.Model)
+    class Exemple2:
+        id = Integer(primary_key=True)
+        exemple = One2One(model='Model.Exemple', backref='exemple2')
+
+
+@pytest.fixture(scope="class")
+def registry_field_one2one(request, bloks_loaded):
+    registry = init_registry(add_field_one2one)
+    request.addfinalizer(registry.close)
+    return registry
+
+
+class TestRelationShipOne2One(SchemaBehaviour):
+
+    @pytest.fixture(autouse=True)
+    def transact(self, request, registry_field_one2one):
+        transaction = registry_field_one2one.begin_nested()
+        request.addfinalizer(transaction.rollback)
+
+    def test_dump_one2one_1(self, registry_field_one2one):
+        registry = registry_field_one2one
         exemple_schema = self.getExempleSchema()(registry=registry)
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         data = exemple_schema.dump(exemple)
-        self.assertEqual(
-            data,
+        assert (
+            data ==
             {
                 'id': exemple.id,
                 'exemple2': {
@@ -78,14 +73,14 @@ class TestRelationShip(DBTestCase):
             }
         )
 
-    def test_dump_one2one_2(self):
-        registry = self.init_registry(self.add_field_one2one)
+    def test_dump_one2one_2(self, registry_field_one2one):
+        registry = registry_field_one2one
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         data = exemple2_schema.dump(exemple2)
-        self.assertEqual(
-            data,
+        assert (
+            data ==
             {
                 'id': exemple.id,
                 'exemple': {
@@ -94,8 +89,8 @@ class TestRelationShip(DBTestCase):
             }
         )
 
-    def test_load_one2one_1(self):
-        registry = self.init_registry(self.add_field_one2one)
+    def test_load_one2one_1(self, registry_field_one2one):
+        registry = registry_field_one2one
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -106,10 +101,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple_schema = self.getExempleSchema()(registry=registry)
         data = exemple_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_load_one2one_2(self):
-        registry = self.init_registry(self.add_field_one2one)
+    def test_load_one2one_2(self, registry_field_one2one):
+        registry = registry_field_one2one
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -120,10 +115,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         data = exemple2_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_validate_one2one_1(self):
-        registry = self.init_registry(self.add_field_one2one)
+    def test_validate_one2one_1(self, registry_field_one2one):
+        registry = registry_field_one2one
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -134,10 +129,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple_schema = self.getExempleSchema()(registry=registry)
         errors = exemple_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
 
-    def test_validate_one2one_2(self):
-        registry = self.init_registry(self.add_field_one2one)
+    def test_validate_one2one_2(self, registry_field_one2one):
+        registry = registry_field_one2one
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -148,10 +143,37 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         errors = exemple2_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
 
-    def test_load_one2many(self):
-        registry = self.init_registry(self.add_field_one2many)
+
+def add_field_one2many():
+
+    @Declarations.register(Declarations.Model)
+    class Exemple:
+        id = Integer(primary_key=True)
+
+    @Declarations.register(Declarations.Model)
+    class Exemple2:
+        id = Integer(primary_key=True)
+        exemple = Many2One(model='Model.Exemple', one2many='exemple2')
+
+
+@pytest.fixture(scope="class")
+def registry_field_one2many(request, bloks_loaded):
+    registry = init_registry(add_field_one2many)
+    request.addfinalizer(registry.close)
+    return registry
+
+
+class TestRelationShipOne2Many(SchemaBehaviour):
+
+    @pytest.fixture(autouse=True)
+    def transact(self, request, registry_field_one2many):
+        transaction = registry_field_one2many.begin_nested()
+        request.addfinalizer(transaction.rollback)
+
+    def test_load_one2many(self, registry_field_one2many):
+        registry = registry_field_one2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -164,16 +186,16 @@ class TestRelationShip(DBTestCase):
         }
         exemple_schema = self.getExempleSchema()(registry=registry)
         data = exemple_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_dump_one2many(self):
-        registry = self.init_registry(self.add_field_one2many)
+    def test_dump_one2many(self, registry_field_one2many):
+        registry = registry_field_one2many
         exemple_schema = self.getExempleSchema()(registry=registry)
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         data = exemple_schema.dump(exemple)
-        self.assertEqual(
-            data,
+        assert (
+            data ==
             {
                 'id': exemple.id,
                 'exemple2': [
@@ -184,8 +206,8 @@ class TestRelationShip(DBTestCase):
             }
         )
 
-    def test_validate_one2many(self):
-        registry = self.init_registry(self.add_field_one2many)
+    def test_validate_one2many(self, registry_field_one2many):
+        registry = registry_field_one2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -198,10 +220,18 @@ class TestRelationShip(DBTestCase):
         }
         exemple_schema = self.getExempleSchema()(registry=registry)
         errors = exemple_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
 
-    def test_load_many2one(self):
-        registry = self.init_registry(self.add_field_one2many)
+
+class TestRelationShipMany2One(SchemaBehaviour):
+
+    @pytest.fixture(autouse=True)
+    def transact(self, request, registry_field_one2many):
+        transaction = registry_field_one2many.begin_nested()
+        request.addfinalizer(transaction.rollback)
+
+    def test_load_many2one(self, registry_field_one2many):
+        registry = registry_field_one2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -212,16 +242,16 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         data = exemple2_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_dump_many2one(self):
-        registry = self.init_registry(self.add_field_one2many)
+    def test_dump_many2one(self, registry_field_one2many):
+        registry = registry_field_one2many
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         data = exemple2_schema.dump(exemple2)
-        self.assertEqual(
-            data,
+        assert (
+            data ==
             {
                 'id': exemple.id,
                 'exemple': {
@@ -230,8 +260,8 @@ class TestRelationShip(DBTestCase):
             }
         )
 
-    def test_validate_many2one(self):
-        registry = self.init_registry(self.add_field_one2many)
+    def test_validate_many2one(self, registry_field_one2many):
+        registry = registry_field_one2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert(exemple=exemple)
         dump_data = {
@@ -242,17 +272,44 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         errors = exemple2_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
 
-    def test_dump_many2many_1(self):
-        registry = self.init_registry(self.add_field_many2many)
+
+def add_field_many2many():
+
+    @Declarations.register(Declarations.Model)
+    class Exemple:
+        id = Integer(primary_key=True)
+
+    @Declarations.register(Declarations.Model)
+    class Exemple2:
+        id = Integer(primary_key=True)
+        exemple = Many2Many(model='Model.Exemple', many2many='exemple2')
+
+
+@pytest.fixture(scope="class")
+def registry_field_many2many(request, bloks_loaded):
+    registry = init_registry(add_field_many2many)
+    request.addfinalizer(registry.close)
+    return registry
+
+
+class TestRelationShipMany2Many(SchemaBehaviour):
+
+    @pytest.fixture(autouse=True)
+    def transact(self, request, registry_field_many2many):
+        transaction = registry_field_many2many.begin_nested()
+        request.addfinalizer(transaction.rollback)
+
+    def test_dump_many2many_1(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple_schema = self.getExempleSchema()(registry=registry)
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
         data = exemple_schema.dump(exemple)
-        self.assertEqual(
-            data,
+        assert (
+            data ==
             {
                 'id': exemple.id,
                 'exemple2': [
@@ -263,15 +320,15 @@ class TestRelationShip(DBTestCase):
             }
         )
 
-    def test_dump_many2many_2(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_dump_many2many_2(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
         data = exemple2_schema.dump(exemple2)
-        self.assertEqual(
-            data,
+        assert (
+            data ==
             {
                 'id': exemple.id,
                 'exemple': [
@@ -282,8 +339,8 @@ class TestRelationShip(DBTestCase):
             }
         )
 
-    def test_load_many2many_1(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_load_many2many_1(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
@@ -297,10 +354,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple_schema = self.getExempleSchema()(registry=registry)
         data = exemple_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_load_many2many_2(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_load_many2many_2(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
@@ -314,10 +371,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         data = exemple2_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_load_many2many_3(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_load_many2many_3(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
@@ -327,10 +384,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         data = exemple2_schema.load(dump_data)
-        self.assertEqual(data, dump_data)
+        assert data == dump_data
 
-    def test_validate_many2many_1(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_validate_many2many_1(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
@@ -344,10 +401,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple_schema = self.getExempleSchema()(registry=registry)
         errors = exemple_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
 
-    def test_validate_many2many_2(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_validate_many2many_2(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
@@ -361,10 +418,10 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         errors = exemple2_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
 
-    def test_validate_many2many_3(self):
-        registry = self.init_registry(self.add_field_many2many)
+    def test_validate_many2many_3(self, registry_field_many2many):
+        registry = registry_field_many2many
         exemple = registry.Exemple.insert()
         exemple2 = registry.Exemple2.insert()
         exemple2.exemple.append(exemple)
@@ -374,4 +431,4 @@ class TestRelationShip(DBTestCase):
         }
         exemple2_schema = self.getExemple2Schema()(registry=registry)
         errors = exemple2_schema.validate(dump_data)
-        self.assertFalse(errors)
+        assert not errors
